#!/usr/bin/env python
# -*- coding: utf-8 -*-

# Aufrufen mit 'python3 Hoerspielland-HLP.py'

import subprocess
import os
from mfrc522 import SimpleMFRC522
from gpiozero import LED, Button
import time
import random
import json
import urllib.request
import re
import math
import socket

print("-- Hörspielland Play --")
print("Version 1.29")

WithDebugMsg = True                     # Zusätzliche Ausgabe von erweiterten Debug-Meldungen
AudioHeading = True                     # Schritte ansagen
WithOLEDDisplay = True                  # OLED Display ansprechen

PathToSkript = os.getcwd()
PathToAdminAudio = PathToSkript + "/HLP-Admin"
PathToPlayAudio = PathToSkript + "/HLP-Audio"
PathToMPC = "/usr/bin"
FilenameDatabase = {"RFID": "HLP-RFID-Database.txt", "Shuffle": "HLP-Shuffle-Database.txt", "Audiobook": "HLP-Audiobook-Database.txt", "Album": "HLP-Album-Database.txt" }


if (WithOLEDDisplay):
        try:
                from lib_oled96 import ssd1306
                from PIL import ImageFont, ImageDraw, Image
                font = ImageFont.load_default()
                from smbus import SMBus
                i2cbus = SMBus(1)                        #
                oled = ssd1306(i2cbus)
                draw = oled.canvas
                # Load default font.
        #        font = ImageFont.load_default() 
                font = ImageFont.truetype('FreeSans.ttf', 15)
        except:
                WithOLEDDisplay = False

# Um RFID-Karten aus der WebApp direkt aufzurufen wird Socket verwendet:
WebApp_UDP_Quelle = '127.0.0.1'
WebApp_UDP_Port = 9009
WebApp_UDP_Socket = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)
WebApp_UDP_Socket.bind( (WebApp_UDP_Quelle, WebApp_UDP_Port) )
WebApp_UDP_Socket.setblocking(0)


Mode_Idle = 0
Mode_Play = 1
Mode_Register = 2
Mode_Delete = 3
Mode_Admin = 4
HLPMode = Mode_Idle
HLPModes = ["Idle", "Play", "Register", "Delete", "Admin"]

RegisterMode_Dir = 1
RegisterMode_File = 2
RegisterMode_PlayMode = 3
RegisterMode = RegisterMode_Dir

PlayMode_Idle = 0
PlayMode_ShuffleDir = 1
PlayMode_Album = 2
PlayMode_File = 3
PlayMode_Max = 3       # Maximal mögliche Aspielmodi
ChoosePlayMode = PlayMode_Idle
PlayMode = PlayMode_Idle
PlayModes = ["", "Shuffle", "Album", "File"]

MPCPause = 1
MPCPlay = 2
StatusMPC = MPCPause

StartVolume = 50
VolumeCur = StartVolume
VolumeMin = 0
VolumeMax = 100

CurDirNr = -1
CurFileNr = -1
CurPlayDirname = ""
CurPlayFilename = ""
CurDir_list = []
HspList = []

ActFolge = ""
ActSerie = ""
ActPlayType = ""
ActSerienNrOrPlayName = ""

t1 = 0
delta = 0

LogoNr = 0
LogoAnz = 3
ClearScreenAfter = 600


def file_put_contents(filename, data):
        with open(filename, 'w') as f:
                f.write(data)

def file_get_contents(filename):
        data = ""
        if (filename.find('://') > 0):
                try:
                        f = urllib.request.urlopen(filename, None, 10)
                        data = f.read().decode('utf-8')
                except urllib.error.HTTPError:
                        print("URL [" + filename +"] konnte nicht geladen werden.", urllib.error.HTTPError)
                except:
                        print("URL [" + filename +"] konnte nicht geladen werden. WLAN?")
        else:
                if (os.path.isfile(filename)):
                        with open(filename, 'r') as f:
                                data = f.read()
        return data

def readDatabase(DatabaseType):
        global FilenameDatabase
        print("Lese " + DatabaseType + "-Datenbank")
        DatabaseLists = {}
        data = file_get_contents(FilenameDatabase[DatabaseType])
        if (data):
                DatabaseLists = json.loads(data)
        return DatabaseLists

def writeDatabase(DatabaseType, DatabaseLists):
        global FilenameDatabase
        print("Schreibe " + DatabaseType + "-Datenbank")
        data = json.dumps(DatabaseLists)
        file_put_contents(FilenameDatabase[DatabaseType], data)

def showClear():
        global tOLED
        if (WithOLEDDisplay):
                draw.rectangle((0, 0, oled.width-1, oled.height-1), outline=0, fill=0)
                oled.display()

def showLogo():
        global LogoNr, tOLED
        if (WithOLEDDisplay):
                LogoNr += 1
                if (LogoNr > LogoAnz):
                        LogoNr = 1
                draw.rectangle((0, 0, oled.width-1, oled.height-1), outline=255, fill=1)
                logo = Image.open('/home/pi/Hoerspielland-Play/Hoerspielland-Play-Logo-' + str(LogoNr) + '.png')
                draw.bitmap((0, 0), logo, fill=0)
                oled.display()
#                tOLED = time.time()

def showMsg(Zeile1 = "", Zeile2 = "", Zeile3 = "", Zeile4 = ""):
        global RegisterMode, tOLED
        if (WithOLEDDisplay):
                draw.rectangle((0, 0, oled.width-1, oled.height-1), outline=0, fill=0)
                font = ImageFont.truetype('FreeSans.ttf', 14)
                if (Zeile1 == ""):
                        Zeile1 = "Hörspielland Play"
                draw.text((0, 0), Zeile1, font=font, fill=1)
                draw.text((0, 17), Zeile2, font=font, fill=1)
                draw.text((0, 32), Zeile3, font=font, fill=1)
                draw.text((0, 47), Zeile4, font=font, fill=1)
                draw.line((0, 15, oled.width-1, 15), fill=1)
                oled.display()
                tOLED = time.time()

symPlay = 1
symPause = 2
symFFWD = 3
symFRWD = 4
symVolume = 5
symLoading = 6

def showSymbol(sym):
        global tOLED
        if (WithOLEDDisplay):
                draw.rectangle((0, 0, oled.width-1, oled.height-1), outline=0, fill=0)
                # xy bezieht sich auf die linke obere Ecke: (0/0) ist exakt links oben, y ist nach unten hin positiv
                if (sym == symPlay):
                        x1 = round((oled.width * 0.3) / 2)
                        y1 = 16
                        x2 = oled.width - x1
                        y2 = round(y1 + ((oled.height - y1) / 2))
                        x3 = x1
                        y3 = oled.height - 1
                        draw.polygon([(x1,y1), (x2,y2), (x3,y3)], outline=1, fill=1)
                elif (sym == symPause):
                        w = round(oled.width * 0.1)
                        x1 = round(oled.width / 2) - w - round(oled.width * 0.02)
                        x2 = round(oled.width / 2) + round(oled.width * 0.02)
                        draw.rectangle((x1, 0, x1+w, 16-1), outline=1, fill=1)
                        draw.rectangle((x2, 0, x2+w, 16-1), outline=1, fill=1)
                        # Aktuelle Position bezogen auf die gesamte Playlist anzeigen
                        p = subprocess.Popen([PathToMPC + "/mpc", "-f", "%file%"], stdout=subprocess.PIPE)
                        p.wait()
                        antwort = p.stdout.read()
                        antwortStr = antwort.decode("unicode_escape")
                        m = re.search("#(\d+)/\d+ +(\d+):(\d+)/\d+:\d+", antwortStr)
                        if (m):
                                g = m.groups()
                                ActPlaylistTrackNr = int(g[0])
                                ActTrackSekunden = (int(g[1]) * 60) + int(g[2])
                                p = subprocess.Popen([PathToMPC + "/mpc", "-f", "%time%__%file%", "playlist"], stdout=subprocess.PIPE)
                                p.wait()
                                antwort = p.stdout.read()
                                antwortStr = antwort.decode("unicode_escape")
                                zeilen = antwortStr.split("\n")
                                PositionSekunden = 0
                                PlaylistTrackNr = 0
                                for zeile in zeilen:
                                        if (zeile):
                                                PlaylistTrackNr += 1
                                                zeit, TrackFile = zeile.split("__")
                                                m, s = zeit.split(":")
                                                Sekunden = int(m) * 60 + int(s)
                                                if (PlaylistTrackNr < ActPlaylistTrackNr):
                                                        PositionSekunden += int(Sekunden)
                                                elif (PlaylistTrackNr == ActPlaylistTrackNr):
                                                        PositionSekunden += ActTrackSekunden
                                                        break
                                                else:
                                                        break
                                m = math.floor(PositionSekunden / 60)
                                s = PositionSekunden % 60
                                ActPositionStr = ""
                                if (m < 10):
                                        ActPositionStr = "0"
                                ActPositionStr = ActPositionStr + str(m) + ":"
                                if (s < 10):
                                        ActPositionStr = ActPositionStr + "0" 
                                ActPositionStr = ActPositionStr + str(s)
                                font = ImageFont.truetype('FreeSans.ttf', 48)
                                draw.text((5, 17), ActPositionStr, font=font, fill=1)
                elif (sym == symFFWD):
                        w = round(oled.width / 2) -2
                        x1 = 0
                        y1 = 0
                        x2 = round(oled.width / 2)
                        yM = round(oled.height / 2)
                        y3 = oled.height - 1
                        draw.polygon([(x1,y1), (x1+w,yM), (x1,y3)], outline=1, fill=1)
                        draw.polygon([(x2,y1), (x2+w,yM), (x2,y3)], outline=1, fill=1)
                elif (sym == symFRWD):
                        w = round(oled.width / 2) -2
                        x1 = w
                        y1 = 0
                        x2 = round(oled.width / 2) + w
                        yM = round(oled.height / 2)
                        y3 = oled.height - 1
                        draw.polygon([(x1,y1), (x1-w,yM), (x1,y3)], outline=1, fill=1)
                        draw.polygon([(x2,y1), (x2-w,yM), (x2,y3)], outline=1, fill=1)
                elif (sym == symVolume):
                        x1 = 0
                        y1 = oled.height - 1
                        x2 = oled.width - 1
                        y2 = y1
                        x3 = x2
                        y3 = round(oled.height / 3)
                        draw.polygon([(x1,y1), (x2,y2), (x3,y3)], outline=1, fill=0)
                        x1 = 0
                        y1 = oled.height - 1
                        x2 = round(oled.width * VolumeCur/100)
                        if (x2 > 0):
                                x2 = x2 - 1
                        y2 = y1
                        x3 = x2
                        y3 = oled.height - round((x2/oled.width) * (oled.height - y3)) - 1
                        draw.polygon([(x1,y1), (x2,y2), (x3,y3)], outline=1, fill=1)
                elif (sym == symLoading):
                        font = ImageFont.truetype('FreeSans.ttf', 38)
                        draw.text((0, 17), "LADEN", font=font, fill=1)

                oled.display()
                tOLED = time.time()

def showPlay(Serie, Folge):
        global tOLED
        if (WithOLEDDisplay):
                Zeile1 = 'Hörspielland Play'
                Zeile2 = Serie
                Zeile3 = Folge
                Zeile4 = ''
                font = ImageFont.truetype('FreeSans.ttf', 14)
                (font_width, font_height) = font.getsize(Zeile3)
                while (font_width > oled.width):
                        Zeile4 = Zeile3[-1] + Zeile4
                        Zeile3 = Zeile3[:-1]
                        (font_width, font_height) = font.getsize(Zeile3)
                draw.rectangle((0, 0, oled.width-1, oled.height-1), outline=0, fill=0)
                draw.text((0, 0), Zeile1, font=font, fill=1)
                draw.text((0, 17), Zeile2, font=font, fill=1)
                draw.text((0, 32), Zeile3, font=font, fill=1)
                draw.text((0, 47), Zeile4, font=font, fill=1)
                draw.line((0, 15, oled.width-1, 15), fill=1)
                oled.display()
                tOLED = time.time()

tNextShow = 0
NextShow = ""

def showToggle():
        global tNextShow, NextShow
        if (NextShow == "Logo"):
                showLogo()
        elif (NextShow == "Title"):
                showPlay(ActSerie, ActFolge)
        tNextShow = 0
        NextShow = ""

def setActSerieUndFolgeFromFile(Filename):
        global ActFolge, ActSerie
        Filename = Filename.replace("file://", "")
        liste = os.path.split(Filename)
        Pfadname = liste[0]
        Pfad, Verzeichnis = os.path.split(Pfadname)
        ActSerie = Verzeichnis
        liste = os.path.splitext(Filename)
        Pfadname = liste[0]
        Pfad, Dateiname = os.path.split(Pfadname)
        ActFolge = Dateiname

def HLP_TextToSpeech(msg):
        if (AudioHeading):
                print(msg)

def HLP_Volume(txt):
        global tNextShow, NextShow
        print("mpc volume " + txt)

        showSymbol(symVolume)
        if (HLPMode == Mode_Play):
                tNextShow = time.time()
                NextShow = "Title"

        p = subprocess.Popen([PathToMPC + "/mpc", "volume", txt], stdout=subprocess.PIPE)
        p.wait()
#        print(p.stdout.read())

def HLP_InitPlayer():
        while (True):
                print("Init: mpc")
                p = subprocess.Popen([PathToMPC + "/mpc"], stdout=subprocess.PIPE, stderr=subprocess.PIPE)
                p.wait()
                antwortErr = p.stderr.read()
                antwortErrStr = antwortErr.decode("unicode_escape")
                if (not "Connection refused" in antwortErrStr):
                        antwort = p.stdout.read()
                        antwortStr = antwort.decode("unicode_escape")
                        break
                # mpd error: Connection refused
                print("Init: mopidy und mpc stehen noch nicht zur Verfügung.")
                time.sleep(2)

        # Wenn der Player bereits läuft, keine Initialisierung durchführen.
        if (not ("[paused]" in antwortStr or "[playing]" in antwortStr)):
                HLP_Volume(str(VolumeCur))
                HLP_PlayAdminFile("0200_intro.mp3")

def HLP_Pause():
        global tNextShow, NextShow
        allLED(False)
        print("mpc pause")
        p = subprocess.Popen([PathToMPC + "/mpc", "pause"], stdout=subprocess.PIPE)
        p.wait()
        showSymbol(symPause)
        tNextShow = time.time()
        NextShow = "Logo"
        if (PlayMode == PlayMode_Album):
                HLP_PlaylistFortschritt()

def HLP_Play():
        global tNextShow, NextShow
        allLED(True)
        showSymbol(symPlay)
        tNextShow = time.time()
        NextShow = "Title"
        print("mpc")
        p = subprocess.Popen([PathToMPC + "/mpc"], stdout=subprocess.PIPE)
        p.wait()
        antwort = p.stdout.read()
        antwortStr = antwort.decode("unicode_escape")
        if ("[playing]" in antwortStr):
                # z.B. bereits über die WebApp gestartet
                print("spielt bereits")
        elif ("[paused]" in antwortStr):
                print("mpc play")
                p = subprocess.Popen([PathToMPC + "/mpc", "play"], stdout=subprocess.PIPE)
                p.wait()
        else:
                # z.B. bereits am Ende der aktuellen Playlist angekommen. Von vorne erneut starten.
                print("mpc play 1")
                p = subprocess.Popen([PathToMPC + "/mpc", "play", "1"], stdout=subprocess.PIPE)
                p.wait()

def HLP_PlayAdminFile(AdminFilename, PlayAfterCurrent = False):
        if (os.path.isfile(PathToAdminAudio + "/" + AdminFilename)):
                if (not PlayAfterCurrent):
                        print("mpc stop")
                        p = subprocess.Popen([PathToMPC + "/mpc", "stop"], stdout=subprocess.PIPE)
                        p.wait()
                        print("mpc clear")
                        p = subprocess.Popen([PathToMPC + "/mpc", "clear"], stdout=subprocess.PIPE)
                        p.wait()
                print("mpc add file://" + PathToAdminAudio + "/" + AdminFilename)
                p = subprocess.Popen([PathToMPC + "/mpc", "add", "file://" + PathToAdminAudio + "/" + AdminFilename], stdout=subprocess.PIPE)
                p.wait()
                if (not PlayAfterCurrent):
                        print("mpc play")
                        p = subprocess.Popen([PathToMPC + "/mpc", "play"], stdout=subprocess.PIPE)
                        p.wait()
        else:
                print("Admin-Audio-File '" + AdminFilename + "' existiert nicht.")

def HLP_PlayFile(PlayFilename, PlayAfterCurrent = False, OneFileOnly = False):
        global StatusMPC, HLPMode
        global tNextShow, NextShow

        if ("hlp-spotify-folge:" in PlayFilename):
                # Album ID von hörspielland holen
                Dummy, TontraegerNr, FolgeName = PlayName.split(':')
                AlbumID = file_get_contents("http://www.hoerspielland.de/getSpotifyPlaylist.php?nr=" + TontraegerNr + "&TstattS=true")
                if (AlbumID):
                        PlayFilename = "spotify:album:" + AlbumID
                else:
                        print("AlbumID konnte nicht von hörspielland.de geholt werden.")

        print("PlayFile: " + PlayFilename)
        if (("spotify:" in PlayFilename) or os.path.isfile(PathToPlayAudio + "/" + PlayFilename) or os.path.isdir(PathToPlayAudio + "/" + PlayFilename)):
                if (HLPMode == Mode_Play):
                        if (not "spotify:" in PlayFilename):
                                setActSerieUndFolgeFromFile(PlayFilename)
                        showPlay(ActSerie, ActFolge)
                if (not PlayAfterCurrent):
                        print("mpc stop")
                        p = subprocess.Popen([PathToMPC + "/mpc", "stop"], stdout=subprocess.PIPE)
                        p.wait()
                        print("mpc clear")
                        p = subprocess.Popen([PathToMPC + "/mpc", "clear"], stdout=subprocess.PIPE)
                        p.wait()
                if ("spotify:" in PlayFilename):
                        print("mpc add " + PlayFilename)
                        p = subprocess.Popen([PathToMPC + "/mpc", "add", PlayFilename], stdout=subprocess.PIPE)
                        p.wait()
                elif (os.path.isfile(PathToPlayAudio + "/" + PlayFilename)):
                        print("mpc add file://" + PathToPlayAudio + "/" + PlayFilename)
                        p = subprocess.Popen([PathToMPC + "/mpc", "add", "file://" + PathToPlayAudio + "/" + PlayFilename], stdout=subprocess.PIPE)
                        p.wait()
                elif (os.path.isdir(PathToPlayAudio + "/" + PlayFilename)):
                        # Alle mp3-Dateien im Unterzeichnis einzeln hinzufügen:
                        u_dir_list = sorted(os.listdir(PathToPlayAudio + "/" + PlayFilename))
                        for u_filename in u_dir_list:
                                if (os.path.isfile(PathToPlayAudio + "/" + PlayFilename + "/" + u_filename)):
                                        u_file_parts = os.path.splitext(u_filename)
                                        if (u_file_parts[1]):
                                                u_endung = u_file_parts[1]
                                                u_endung = u_endung.lower()
                                                if (u_endung.casefold() == ".mp3" or u_endung.casefold() == ".wav" or u_endung.casefold() == ".flac"):
                                                        print("mpc add file://" + PathToPlayAudio + "/" + PlayFilename + "/" + u_filename)
                                                        p = subprocess.Popen([PathToMPC + "/mpc", "add", "file://" + PathToPlayAudio + "/" + PlayFilename + "/" + u_filename], stdout=subprocess.PIPE)
                                                        p.wait()
                                                        if (OneFileOnly):
                                                                break
                if (not PlayAfterCurrent):
                        print("mpc play")
                        StatusMPC = MPCPlay
                        p = subprocess.Popen([PathToMPC + "/mpc", "play"], stdout=subprocess.PIPE)
                        p.wait()
                        showSymbol(symPlay)
                        tNextShow = time.time()
                        NextShow = "Title"
                        allLED(True)
        else:
                HLP_PlayAdminFile("0401_error.mp3")
                HLP_TextToSpeech("Die Datei '" + PlayFilename + "' extistiert nicht.")
                HLPMode = Mode_Idle
                allLED(False)

def HLP_PlayShuffle(PlayName, PlayAfterCurrent = False):
        global HspList, CurFileNr, HLPMode, ActFolge, ActSerie, ActPlayType, ActSerienNrOrPlayName

        print("PlayShuffle: " + PlayName)

        CurPlayFilename = ""

        if ("hlp-spotify-serie" in PlayName):
                # Spotify-Playlist von hörspielland.de laden
                Dummy, SerienNr, SerienName = PlayName.split(':')
                ShuffleLists = readDatabase("Shuffle")
                if (SerienNr in ShuffleLists.get("hlp-spotify-serie", {}).get("ShufflePosition", {}).keys()):
                        print("Nächster Titel aus der Spotify-Shuffle-Liste abspielen.")
                        ShufflePosition = ShuffleLists["hlp-spotify-serie"]["ShufflePosition"][SerienNr]
                        ShufflePosition += 1
                        if (SerienNr in ShuffleLists["hlp-spotify-serie"].get("ShufflePlaylist", {}).keys()):
                                HspList = ShuffleLists["hlp-spotify-serie"]["ShufflePlaylist"][SerienNr]
                                if (len(HspList) >= (ShufflePosition + 1)):
                                        posList = ShuffleLists["hlp-spotify-serie"]["ShufflePlaylist"][SerienNr][ShufflePosition]
                                        if (posList.get("SpotifyAlbumId") and posList.get("Serie") and posList.get("Folge")):
                                                CurPlayFilename = "spotify:album:" + posList["SpotifyAlbumId"]
                                        else:
                                                print("Playlist enthält ungültige Daten.")
                                else:
                                        print("Am Ende der Shuffle-Spotify-Playlist angelangt. Liste wird jetzt neu von hörspielland.de geladen und erstellt.")
                if (not CurPlayFilename):
                        ShufflePosition = 0
                        hlpSpotifyPlaylistData = file_get_contents("http://www.hoerspielland.de/getSpotifyPlaylist.php?nr=" + SerienNr)
                        if (hlpSpotifyPlaylistData):
                                hlpSpotifyPlaylist = json.loads(hlpSpotifyPlaylistData)
                                random.shuffle(hlpSpotifyPlaylist)
                                if (not ShuffleLists.get("hlp-spotify-serie")):
                                        ShuffleLists["hlp-spotify-serie"] = {"ShufflePlaylist": {}, "ShufflePosition": {}}
#                                ShuffleLists["hlp-spotify-serie"]["ShufflePosition"][SerienNr] = ShufflePosition
                                ShuffleLists["hlp-spotify-serie"]["ShufflePlaylist"][SerienNr] = hlpSpotifyPlaylist
                                CurPlayFilename = "spotify:album:" + ShuffleLists["hlp-spotify-serie"]["ShufflePlaylist"][SerienNr][ShufflePosition]["SpotifyAlbumId"]
                        else:
                                print("Playlist konnte von hörspielland.de nicht geladen werden.")
                                showMsg("","Keine Verbindung", "zu", "hörspielland.de")
                                HLP_PlayAdminFile("0401_error.mp3")
                                HLPMode = Mode_Idle
                                allLED(False)
                                return

                ShuffleLists["hlp-spotify-serie"]["ShufflePosition"][SerienNr] = ShufflePosition                
                writeDatabase("Shuffle", ShuffleLists)
                ActPlayType = "hlp-spotify-serie"
                ActSerienNrOrPlayName = SerienNr
                ActSerie = ShuffleLists["hlp-spotify-serie"]["ShufflePlaylist"][SerienNr][ShufflePosition]["Serie"]
                ActFolge = ShuffleLists["hlp-spotify-serie"]["ShufflePlaylist"][SerienNr][ShufflePosition]["Folge"]
                if (WithDebugMsg):
                        print("Serie: " + ActSerie)
                        print("Folge: " + ActFolge)
        else:
                PlayDirname = PlayName
                # PlayDirname ist das Unterverzeichnis in '/home/pi/Hoerspielland-Play/HLP-Audio'
                ShuffleLists = readDatabase("Shuffle")
                if (PlayDirname in ShuffleLists.get("hlp-files", {}).get("ShufflePosition", {}).keys()):
                        print("Nächster Titel aus der Datei-Shuffle-Liste abspielen.")
                        ShufflePosition = ShuffleLists["hlp-files"]["ShufflePosition"][PlayDirname]
                        ShufflePosition += 1
                        if (PlayDirname in ShuffleLists["hlp-files"].get("ShufflePlaylist", {}).keys()):
                                HspList = ShuffleLists["hlp-files"].get("ShufflePlaylist", {}).get(PlayDirname)
                                if (len(HspList) >= (ShufflePosition + 1)):
                                        CurPlayFilename = ShuffleLists["hlp-files"]["ShufflePlaylist"][PlayDirname][ShufflePosition]
                                else:
                                        print("Am Ende der Shuffle-Dateiliste angelangt. Liste wird jetzt neu erstellt.")
                if (not CurPlayFilename):
                        if (os.path.isdir(PathToPlayAudio + "/" + PlayDirname)):
                                HspList = get_HoerspieleImVerzeichnis(PlayDirname)
                                if(HspList):
                                        random.shuffle(HspList)
                                        ShufflePosition = 0
                                        if (not ShuffleLists.get("hlp-files")):
                                                ShuffleLists["hlp-files"] = {"ShufflePlaylist": {}, "ShufflePosition": {}}
                                        ShuffleLists["hlp-files"]["ShufflePosition"][PlayDirname] = ShufflePosition
                                        ShuffleLists["hlp-files"]["ShufflePlaylist"][PlayDirname] = HspList
                                        CurPlayFilename = ShuffleLists["hlp-files"]["ShufflePlaylist"][PlayDirname][ShufflePosition]
                                else:
                                        print("Das Verzeichnis [" + PlayDirname + "] ist leer.")
                        else:
                                print("Verzeichnis [" + PlayDirname + "] existiert nicht.")
                ShuffleLists["hlp-files"]["ShufflePosition"][PlayDirname] = ShufflePosition
                writeDatabase("Shuffle", ShuffleLists)
                ActPlayType = "hlp-files"
                ActSerienNrOrPlayName = PlayName
                setActSerieUndFolgeFromFile(CurPlayFilename)
                if (not (os.path.isfile(PathToPlayAudio + "/" + CurPlayFilename) or os.path.isdir(PathToPlayAudio + "/" + CurPlayFilename))):
                        CurPlayFilename = ""
                        print("Datei/Unterverzeichnis [" + PathToPlayAudio + "/" + CurPlayFilename + "] existiert nicht.")
                else:
                        if (WithDebugMsg):
                                print("Serie: " + ActSerie)
                                print("Folge: " + ActFolge)


        if (CurPlayFilename):
                HLP_PlayFile(CurPlayFilename, PlayAfterCurrent)
        else:
                HLP_PlayAdminFile("0401_error.mp3")
                HLPMode = Mode_Idle
                allLED(False)

def HLP_PlayAudiobook(PlayName):
        global HspList, CurFileNr, StatusMPC
        global ActFolgeNr, ActFolge, ActSerie, ActPlayType, ActPlayName, ActSerienNrOrPlayName
        global tNextShow, NextShow

        # Wird zur Zeit nicht verwendet

        showSymbol(symLoading)
        blinkAllLED()

        ActPlayName = PlayName
        ActPlayType = ""
        ActSerienNrOrPlayName = 0

        print("PlayAudiobook: " + PlayName)
        StatusMPC = MPCPause
        print("mpc stop")
        p = subprocess.Popen([PathToMPC + "/mpc", "stop"], stdout=subprocess.PIPE)
        p.wait()
        print("mpc clear")
        p = subprocess.Popen([PathToMPC + "/mpc", "clear"], stdout=subprocess.PIPE)
        p.wait()

        if ("hlp-spotify-serie" in PlayName):
                # PlayName ist die Spotify-Serienliste von Hörspielland.de
                # Spotify-Playlist von hörspielland.de laden
                Dummy, SerienNr, SerienName = PlayName.split(':')

                AlbumLists = readDatabase("Audiobook")
                if (AlbumLists.get(PlayName)):
                        # Der Fortschritt bezieht sich hier auf die Folge und auf die Sekunden innerhalb einer Folge
                        ActFolgeNr, Sekunden, TrackFile = AlbumLists[PlayName].split("__")
                        try:
                                ActFolgeNr = int(ActFolgeNr)
                                Sekunden = int(Sekunden)
                                print("Fortschritt aus der Database:")
                        except:
                                ActFolgeNr = 0
                                Sekunden = 0
                                TrackFile = ""
                else:
                        ActFolgeNr = 0
                        Sekunden = 0
                        TrackFile = ""
 
                print("ActFolgeNr: " + str(ActFolgeNr))
                print("Sekunden: " + str(Sekunden))

                hlpSpotifyPlaylistData = file_get_contents("http://www.hoerspielland.de/getSpotifyPlaylist.php?nr=" + SerienNr)
                if (WithDebugMsg):
                        print (hlpSpotifyPlaylistData)
                if (hlpSpotifyPlaylistData):
                        hlpSpotifyPlaylist = json.loads(hlpSpotifyPlaylistData)
                        CurPlayFilename = "spotify:album:" + hlpSpotifyPlaylist[ActFolgeNr]["SpotifyAlbumId"]
                        ActSerie = hlpSpotifyPlaylist[ActFolgeNr]["Serie"]
                        ActFolge = hlpSpotifyPlaylist[ActFolgeNr]["Folge"]
                        ActPlayType = "hlp-spotify-serie"
                        ActSerienNrOrPlayName = SerienNr

                        print("mpc add " + CurPlayFilename)
                        p = subprocess.Popen([PathToMPC + "/mpc", "add", CurPlayFilename], stdout=subprocess.PIPE)
                        p.wait()
                        StatusMPC = MPCPlay
                        print("mpc play")
                        p = subprocess.Popen([PathToMPC + "/mpc", "play"], stdout=subprocess.PIPE)
                        p.wait()
                        if (Sekunden > 0):
                                time.sleep(1)
                                print("mpc seekthrough " + str(Sekunden))
                                p = subprocess.Popen([PathToMPC + "/mpc", "seekthrough", str(Sekunden)], stdout=subprocess.PIPE)
                                p.wait()

                        showSymbol(symPlay)
                        tNextShow = time.time()
                        NextShow = "Title"
                        allLED(True)
                else:
                        print("Spotify-Playlist konnte von hörspielland.de nicht geladen werden.")
                        showMsg("","Keine Verbindung", "zu", "hörspielland.de")
                        HLP_PlayAdminFile("0401_error.mp3")
                        HLPMode = Mode_Idle
                        allLED(False)
                        return
        else:
                # PlayName ist das Unterverzeichnis in '/home/pi/Hoerspielland-Play/HLP-Audio'
                # Alle MP3 im Verzeichnis der Reihe nach abspielen und den Fortschritt speichern
                # Handelt es sich um eine konkrete MP3-Datei oder um eine hlp-spotify-folge, dann kann soetwas nicht als Album abgespielt werden und die folgende Prüfung auf ein existierendes Verzeichnis führt zum Fehler.
                if (os.path.isdir(PathToPlayAudio + "/" + PlayName)):
                        # Alle Dateien aus dem Verzeichis hinzufügen
                        print("mpc add ...")
                        HspList = get_HoerspieleImVerzeichnis(PlayName)
                        # Fortschritt einlesen
                        AlbumLists = readDatabase("Audiobook")
                        if (PlayName in AlbumLists):
                                ActFolgeNr, Sekunden, TrackFile = AlbumLists[PlayName].split("__")
                                try:
                                        ActFolgeNr = int(ActFolgeNr)
                                        Sekunden = int(Sekunden)
                                except:
                                        ActFolgeNr = 0
                                        Sekunden = 0
                                        TrackFile = ""
                        else:
                                ActFolgeNr = 0
                                Sekunden = 0
                                TrackFile = ""
                        TrackNr = 0
                        ActTrackNr = 0
                        for PlayFilename in HspList:
                                TrackNr += 1
                                print("mpc add file://" + PathToPlayAudio + "/" + PlayFilename)
                                p = subprocess.Popen([PathToMPC + "/mpc", "add", "file://" + PathToPlayAudio + "/" + PlayFilename], stdout=subprocess.PIPE)
                                p.wait()
                                if ("file://" + PathToPlayAudio + "/" + PlayFilename == TrackFile):
                                        ActTrackNr = TrackNr

                        if (TrackFile and ActTrackNr == 0):
                                print("Das Verzeichnis hat sich geändert. Abspielen von vorn.")
                                setActSerieUndFolgeFromFile(PlayName)
                        else:
                                setActSerieUndFolgeFromFile(TrackFile)
                        if (ActTrackNr == 0):
                                ActTrackNr = 1
                        if (TrackNr > 0):
                                ActPlayType = "hlp-file"
                                ActSerienNrOrPlayName = PlayName
                                StatusMPC = MPCPlay
                                time.sleep(1)
                                print("mpc play " + str(ActTrackNr))
                                p = subprocess.Popen([PathToMPC + "/mpc", "play", str(ActTrackNr)], stdout=subprocess.PIPE)
                                p.wait()
                                if (Sekunden > 0):
                                        time.sleep(1)
                                        print("mpc seekthrough +" + str(Sekunden))
                                        p = subprocess.Popen([PathToMPC + "/mpc", "seekthrough", "+" + str(Sekunden)], stdout=subprocess.PIPE)
                                        p.wait()
                                showSymbol(symPlay)
                                tNextShow = time.time()
                                NextShow = "Title"
                                allLED(True)
                                return  
                        else:
                                print("Das Verzeichnis ist leer.")

                showMsg("", "Keine Dateien", "im Verzeichnis")
                HLP_PlayAdminFile("0401_error.mp3")
                HLP_TextToSpeech("Das Verzeichnis '" + PlayName + "' extistiert nicht oder es enthält keine abspielbare Datei.")
                HLPMode = Mode_Idle
                allLED(False)

def HLP_PlayAlbum(PlayName, Prev = False):
        global HspList, CurFileNr, StatusMPC
        global ActFolgeNr, ActFolge, ActSerie, ActPlayType, ActPlayName, ActSerienNrOrPlayName
        global tNextShow, NextShow

        # Im Album-Modus werden alle Hörspiele der Reihe nach abgespielt. Der Fortschritt innerhalb eines Hörspiels wird gespeichert.
        # Nach Auflegen einer RFID-Karte wird immer nur ein Hörspiel abgespielt.

        doPlay = False
        MsgDone = False

        showSymbol(symLoading)
        blinkAllLED()

        ActPlayName = PlayName
        ActPlayType = ""
        ActSerienNrOrPlayName = 0

        print("PlayAlbum: " + PlayName)
        StatusMPC = MPCPause
        print("mpc stop")
        p = subprocess.Popen([PathToMPC + "/mpc", "stop"], stdout=subprocess.PIPE)
        p.wait()
        print("mpc clear")
        p = subprocess.Popen([PathToMPC + "/mpc", "clear"], stdout=subprocess.PIPE)
        p.wait()

        # Fortschritt einlesen
        AlbumLists = readDatabase("Album")
        if (AlbumLists.get(PlayName)):
                ActFolgeNr, Sekunden, HspFortsetzen = AlbumLists[PlayName].split("__")
                try:
                        ActFolgeNr = int(ActFolgeNr)
                        Sekunden = int(Sekunden)
                        HspFortsetzen = int(HspFortsetzen)
                        print("Fortschritt aus der Database:")
                except:
                        ActFolgeNr = 0          # wird gleich auf 1 gesetzt
                        Sekunden = 0
                        HspFortsetzen = 0       # 0 = nicht fortsetzen / 1 = fortsetzen
        else:
                ActFolgeNr = 0
                Sekunden = 0
                HspFortsetzen = 0

        if (Prev):
                ActFolgeNr -= 1
                Sekunden = 0
                print("Vorheriges Hörspiel abspielen.")
        elif (not HspFortsetzen):
                ActFolgeNr += 1
                Sekunden = 0
                print("Nächstes Hörspiel abspielen.")

        print("ActFolgeNr: " + str(ActFolgeNr))
        print("Ab Sekunde: " + str(Sekunden))

        HspList = []

        if ("hlp-spotify-serie" in PlayName):
                # PlayName ist die Spotify-Serienliste von Hörspielland.de
                # Spotify-Playlist von hörspielland.de laden
                Dummy, SerienNr, SerienName = PlayName.split(':')
                hlpSpotifyPlaylistData = file_get_contents("http://www.hoerspielland.de/getSpotifyPlaylist.php?nr=" + SerienNr)
                if (WithDebugMsg):
                        print (hlpSpotifyPlaylistData)
                if (hlpSpotifyPlaylistData):
                        HspList = hlpSpotifyPlaylist = json.loads(hlpSpotifyPlaylistData)
                else:
                        print("Spotify-Playlist konnte von hörspielland.de nicht geladen werden.")
                        showMsg("","Keine Verbindung", "zu", "hörspielland.de")
                        MsgDone = True
        elif (os.path.isdir(PathToPlayAudio + "/" + PlayName)):
                # PlayName ist ein Unterverzeichnis im Basisverzeichnis
                HspList = get_HoerspieleImVerzeichnis(PlayName)
        else:
                print("Das Verzeichnis '" + PlayName + "' existiert nicht oder ungültiger PlayName der RFID-Karte.")

        if (HspList):
                if (len(HspList) < ActFolgeNr):
                        ActFolgeNr = 1  # Zählung beginnt bei 1
                        Sekunden = 0
                        print("Am Ende angelangt. Wieder das erste Hörspiel abspielen.")
                elif (ActFolgeNr < 1):
                        ActFolgeNr = len(HspList)
                        Sekunden = 0
                        print("Am Anfang angelangt. Jetzt das letzte Hörspiel abspielen.")
                if ("hlp-spotify-serie" in PlayName):
                        CurPlayFilename = "spotify:album:" + hlpSpotifyPlaylist[(ActFolgeNr - 1)]["SpotifyAlbumId"]
                        ActSerie = hlpSpotifyPlaylist[(ActFolgeNr - 1)]["Serie"]
                        ActFolge = hlpSpotifyPlaylist[(ActFolgeNr - 1)]["Folge"]
                        ActPlayType = "hlp-spotify-serie"
                        ActSerienNrOrPlayName = SerienNr

                        print("mpc add " + CurPlayFilename)
                        p = subprocess.Popen([PathToMPC + "/mpc", "add", CurPlayFilename], stdout=subprocess.PIPE)
                        p.wait()
                        doPlay = True
                else:
                        HspFile = HspList[(ActFolgeNr - 1)]
                        # HspFile enthält bereits PlayName
                        setActSerieUndFolgeFromFile(HspFile)

                        # Handelt es sich bei HspFile um eine Datei oder um ein Verzeichnis?
                        if (os.path.isfile(PathToPlayAudio + "/" + HspFile)):
                                print("mpc add file://" + PathToPlayAudio + "/" + HspFile)
                                p = subprocess.Popen([PathToMPC + "/mpc", "add", "file://" + PathToPlayAudio + "/" + HspFile], stdout=subprocess.PIPE)
                                p.wait()
                                doPlay = True
                        elif (os.path.isdir(PathToPlayAudio + "/" + HspFile)):
                                # Das Hörspiel besteht aus Tracks.
                                # Alle Tracks aus dem Verzeichnis als Playlist laden.
                                HspList = get_HoerspieleImVerzeichnis(HspFile)
                                a = 0
                                for PlayFilename in HspList:
                                        if (os.path.isfile(PathToPlayAudio + "/" + PlayFilename)):
                                                a += 1
                                                print("mpc add file://" + PathToPlayAudio + "/" + PlayFilename)
                                                p = subprocess.Popen([PathToMPC + "/mpc", "add", "file://" + PathToPlayAudio + "/" + PlayFilename], stdout=subprocess.PIPE)
                                                p.wait()
                                if (a == 0):
                                        HLP_TextToSpeech("Das Unterverzeichnis '" + HspFile + "' ist leer.")
                                else:
                                        doPlay = True
                        else:
                                HLP_TextToSpeech("'" + HspFile + "' existiert nicht.")
        else:
                HLP_TextToSpeech("Das Verzeichnis '" + PlayName + "' ist leer.")

        AlbumLists[PlayName] = str(ActFolgeNr) + "__" + str(Sekunden) + "__0"
        writeDatabase("Album", AlbumLists) 

        if (doPlay):
                time.sleep(1)
                print("mpc play")
                p = subprocess.Popen([PathToMPC + "/mpc", "play"], stdout=subprocess.PIPE)
                p.wait()
                if (Sekunden > 0):
                        time.sleep(1)
                        print("mpc seekthrough +" + str(Sekunden))
                        p = subprocess.Popen([PathToMPC + "/mpc", "seekthrough", "+" + str(Sekunden)], stdout=subprocess.PIPE)
                        p.wait()
                StatusMPC = MPCPlay
                showSymbol(symPlay)
                tNextShow = time.time()
                NextShow = "Title"
                allLED(True)
        else:
                if (not MsgDone):
                        showMsg("", "Keine Dateien", "im Verzeichnis")
                HLP_PlayAdminFile("0401_error.mp3")
                HLPMode = Mode_Idle
                allLED(False)

def HLP_PlaylistFortschritt():
        global ActFolgeNr, ActPlayName

        # Fortschritt beim Abspielen der aktuellen Playlist speichern
        # mpc -f "%file%"
        # file:///home/pi/Hoerspielland-Play/HLP-Audio/xAlf/001.mp3
        # [playing] #1/3   0:13/18:47 (1%)
        # volume:100%   repeat: off   random: off   single: off   consume: off

        if (ActPlayName):
                AlbumLists = readDatabase("Album")
                Sekunden = 0
                p = subprocess.Popen([PathToMPC + "/mpc", "-f", "%file%"], stdout=subprocess.PIPE)
                p.wait()
                antwort = p.stdout.read()
                antwortStr = antwort.decode("unicode_escape")
                m = re.search("#(\d+)/\d+ +(\d+):(\d+)/\d+:\d+", antwortStr)
                if (m):
                        g = m.groups()
                        Sekunden = int(g[1]) * 60 + int(g[2])
                        ActTrackNr = int(g[0])
                        if (ActTrackNr > 1):
                                # Die Sekunden der vorigen Tracks hinzurechnen
                                p = subprocess.Popen([PathToMPC + "/mpc", "-f", "%duration%__%file%", "playlist"], stdout=subprocess.PIPE)
                                p.wait()
                                antwort = p.stdout.read()
                                antwortStr = antwort.decode("unicode_escape")
                                Zeilen = antwortStr.split("\n")
                                t = 0
                                for Zeile in Zeilen:
                                        t += 1
                                        if (t == ActTrackNr):
                                                break
                                        m, s = Zeile.split("__")
                                        Sekunden += int(m) * 60 + int(s)
                        print("Fortschritt merken für [" + str(ActFolgeNr) + "]: " + str(Sekunden) + " Sekunden")
                        AlbumLists[ActPlayName] = str(ActFolgeNr) + "__" + str(Sekunden) + "__1"
                        writeDatabase("Album", AlbumLists)
        else:
                print("Fortschritt kann nicht gespeichert werden!")

def do_PlayAdminMenu(MenuId, PlayAfterCurrent = False):
        print("PlayAdminMenue: " + str(MenuId))
        if (MenuId == PlayMode_ShuffleDir):
                HLP_PlayAdminFile("0311_mode_random_episode.mp3", PlayAfterCurrent)
                HLP_TextToSpeech("Hörspiel-Modus: Zufällige Wiedergabe eines Titels aus diesem Verzeichnis.")
        elif (MenuId == PlayMode_Album):
                HLP_PlayAdminFile("0315_mode_audio_book.mp3", PlayAfterCurrent)
                HLP_TextToSpeech("Hörbuch-Modus: Alle Dateien in einem Verzeichnis der Reihe nach abspielen.")
        elif (MenuId == PlayMode_File):
                HLP_PlayAdminFile("0314_mode_single_track.mp3", PlayAfterCurrent)
                HLP_TextToSpeech("Datei-Modus: Nur exakt eine Datei aus einem Verzeichnis abspielen.")

def do_RFID_Write(text):
        print("Schreibe RFID: " + text)
        reader.write(text)

def do_AbortDelete():
        global HLPMode
        HLP_PlayAdminFile("0802_reset_aborted.mp3")
        HLPMode = Mode_Idle
        HLP_TextToSpeech("Löschen einer Karte abgebrochen.")

def do_FastForward():
        global ActSerie, ActFolge, ActFolgeNr, ActPlayName
        if (PlayMode == PlayMode_ShuffleDir):
                # In der Shuffle-Datenbank wurde die zufällige Reihenfolge der Hörspiele eines Verzeichnis notiert.
                # Springe nun zum nächsten Hörspiel in der Shuffle-Reihenfolge.
                ShuffleLists = readDatabase("Shuffle")
                if (ActSerienNrOrPlayName in ShuffleLists.get(ActPlayType, {}).get("ShufflePosition", {}).keys()):
                        print("Nächsten Titel aus der Shuffle-Liste abspielen.")
                        ShufflePosition = ShuffleLists[ActPlayType]["ShufflePosition"][ActSerienNrOrPlayName]
                        ShufflePosition += 1
                        if (ActSerienNrOrPlayName in ShuffleLists[ActPlayType].get("ShufflePlaylist", {}).keys()):
                                HspList = ShuffleLists[ActPlayType]["ShufflePlaylist"][ActSerienNrOrPlayName]
                                if (len(HspList) >= (ShufflePosition + 1)):
                                        posList = ShuffleLists[ActPlayType]["ShufflePlaylist"][ActSerienNrOrPlayName][ShufflePosition]
                                        if (posList.get("SpotifyAlbumId") and posList.get("Serie") and posList.get("Folge")):
                                                CurPlayFilename = "spotify:album:" + posList["SpotifyAlbumId"]
                                                ActSerie = posList["Serie"]
                                                ActFolge = posList["Folge"]
                                                if (WithDebugMsg):
                                                        print("Serie: " + ActSerie)
                                                        print("Folge: " + ActFolge)
                                                ShuffleLists[ActPlayType]["ShufflePosition"][ActSerienNrOrPlayName] = ShufflePosition                
                                                writeDatabase("Shuffle", ShuffleLists)
                                                HLP_PlayFile(CurPlayFilename, False)
                                        else:
                                                print("Shuffle-Playlist enthält ungültige Daten.")
                                else:
                                        print("Am Ende der Shuffle-Playlist angelangt. Vorspulen nicht möglich.")

        elif (PlayMode == PlayMode_Album):
                # Alle Hörspiele im Verzeichnis sollen der Reihe nach abgespielt werden, dabei wird der Fortschritt gespeichert.
                # Springe nun zum nächsten Hörspiel im Verzeichnis.
                print("ActPlayName: " + ActPlayName)
                if (ActPlayName):
                        HLP_PlayAlbum(ActPlayName)
        elif (PlayMode == PlayMode_File):
                # Es wird exakt ein Hörspiel abgespielt.
                # Spule im aktuell laufenden Hörspiel 5 Minuten vor.
                print("mpc seekthrough +300")
                p = subprocess.Popen([PathToMPC + "/mpc", "seekthrough", "+300"], stdout=subprocess.PIPE)
                p.wait()

def do_FastRewind():
        global ActSerie, ActFolge, ActFolgeNr, ActPlayName
        if (PlayMode == PlayMode_ShuffleDir):
                # Springe nun zum vorigen Hörspiel in der Shuffle-Reihenfolge.
                ShuffleLists = readDatabase("Shuffle")
                if (ActSerienNrOrPlayName in ShuffleLists.get(ActPlayType, {}).get("ShufflePosition", {}).keys()):
                        print("Vorigen Titel aus der Spotify-Shuffle-List abspielen.")
                        ShufflePosition = ShuffleLists[ActPlayType]["ShufflePosition"][ActSerienNrOrPlayName]
                        ShufflePosition -= 1
                        if (ActSerienNrOrPlayName in ShuffleLists[ActPlayType].get("ShufflePlaylist", {}).keys()):
                                HspList = ShuffleLists[ActPlayType]["ShufflePlaylist"][ActSerienNrOrPlayName]
                                if (len(HspList) > 0 and ShufflePosition >= 0):
                                        posList = ShuffleLists[ActPlayType]["ShufflePlaylist"][ActSerienNrOrPlayName][ShufflePosition]
                                        if (posList.get("SpotifyAlbumId") and posList.get("Serie") and posList.get("Folge")):
                                                CurPlayFilename = "spotify:album:" + posList["SpotifyAlbumId"]
                                                ActSerie = posList["Serie"]
                                                ActFolge = posList["Folge"]
                                                if (WithDebugMsg):
                                                        print("Serie: " + ActSerie)
                                                        print("Folge: " + ActFolge)
                                                ShuffleLists[ActPlayType]["ShufflePosition"][ActSerienNrOrPlayName] = ShufflePosition                
                                                writeDatabase("Shuffle", ShuffleLists)
                                                HLP_PlayFile(CurPlayFilename, False)
                                        else:
                                                print("Shuffle-Playlist enthält ungültige Daten.")
                                else:
                                        print("Am Anfang der Shuffle-Playlist angelangt. Zurückspulen nicht möglich.")
        elif (PlayMode == PlayMode_Album):
                # Springe nun zum vorigen Hörspiel im Verzeichnis.
                print("ActPlayName: " + ActPlayName)
                if (ActPlayName):
                        HLP_PlayAlbum(ActPlayName, True)
        elif (PlayMode == PlayMode_File):
                # Spule im aktuell laufenden Hörspiel 5 Minuten zurück.
                print("mpc seekthrough -300")
                p = subprocess.Popen([PathToMPC + "/mpc", "seekthrough", "-300"], stdout=subprocess.PIPE)
                p.wait()


bounceHeldDown = False
bounceHeldPlay = False
bounceHeldUp = False

def do_ButtonDown():
        global HLPMode, RegisterMode, ChoosePlayMode, VolumeCur
        global CurPlayDirname, CurDirNr, CurDir_list, CurPlayFilename, CurFileNr, HspList, PathToAdminAudio, PathToPlayAudio
        global bounceHeldDown
        if (bounceHeldDown):
                bounceHeldDown = False
                return
        print("Button Down gedrückt.")
        print("HLPMode = " + HLPModes[HLPMode])
        if (HLPMode == Mode_Delete):
                do_AbortDelete()
        elif (HLPMode == Mode_Idle or HLPMode == Mode_Play):
                VolumeCur -= 5
                if (VolumeCur < VolumeMin):
                        VolumeCur = VolumeMin
                HLP_Volume(str(VolumeCur))
        elif (HLPMode == Mode_Register):
                if (RegisterMode == RegisterMode_Dir):
                        #Voriges Verzeichnis auswählen und die erste Datei daraus abspielen
                        print("Voriges Verzeichnis anspielen")
                        print("CurDir_list hat " + str(len(CurDir_list)) + " Einträge")
                        print("CurDirNr: " + str(CurDirNr))
                        if (CurDirNr > 0):
                                CurDirNr -= 1
                        else:
                                CurDirNr = len(CurDir_list) - 1
                        CurPlayDirname = CurDir_list[CurDirNr]
                        HspList = get_HoerspieleImVerzeichnis(CurPlayDirname)
                        HLP_PlayFile(HspList[0])
                elif (RegisterMode == RegisterMode_File):
                        #Vorige Datei auswählen und abspielen
                        print("Vorige Datei anspielen")
                        if (CurFileNr > 0):
                                CurFileNr -= 1
                        else:
                                CurFileNr = len(HspList) - 1
                        CurPlayFilename = HspList[CurFileNr]
                        HLP_PlayFile(HspList[CurFileNr])
                elif (RegisterMode == RegisterMode_PlayMode):
                        ChoosePlayMode -= 1
                        if (ChoosePlayMode < 1):
                                ChoosePlayMode = PlayMode_Max
                        do_PlayAdminMenu(ChoosePlayMode)

def do_ButtonDownHeld():
        global CurFileNr
        global bounceHeldDown
        bounceHeldDown = True
        print("Button Down Held gedrückt.")
        if (HLPMode == Mode_Play):
                do_FastRewind()

def do_ButtonPlay():
        global HLPMode, RegisterMode, ChoosePlayMode, CurDirNr, HspList, CurPlayFilename, StatusMPC
        global bounceHeldPlay
        global tNextShow, NextShow
        if (bounceHeldPlay):
                bounceHeldPlay = False
                return
        print("Button Play gedrückt.")
        print("HLPMode = " + HLPModes[HLPMode])
        if (HLPMode == Mode_Play or HLPMode == Mode_Idle):
                # Wurde ein Hörspiel über die WebApp gestartet, dann ist die Box noch im Idle-Modus
                # Ein Tastendruck auf Play schaltet die LEDs an. Ein laufendes Hörspiel wird weiter gespielt.
                if (StatusMPC == MPCPlay):
                        StatusMPC = MPCPause
                        HLP_Pause()
                else:
                        StatusMPC = MPCPlay
                        HLP_Play()
        elif (HLPMode == Mode_Register):
                if (RegisterMode == RegisterMode_Dir):
                        if (CurDirNr > -1):
                                RegisterMode = RegisterMode_PlayMode
                                showMsg("", "Neue Karte:", "Abspielmodus wählen")
                                HLP_PlayAdminFile("0310_tag_linked.mp3")
                                HLP_TextToSpeech("Wähle den Abspielmodus mit den Laut- und Leise-Tasten. Bestätige deine Auswahl mit der OK-Taste. Drücke die OK-Taste lange um den Vorgang abzubrechen.")
                                ChoosePlayMode = PlayMode_Idle
#                                ChoosePlayMode = PlayMode_ShuffleDir
#                                do_PlayAdminMenu(ChoosePlayMode, True)
                elif (RegisterMode == RegisterMode_PlayMode):
                        if (ChoosePlayMode == PlayMode_File):
                                RegisterMode = RegisterMode_File
                                showMsg("", "Neue Karte:", "Datei wählen")
                                HLP_PlayAdminFile("0320_select_file.mp3")
                                HLP_TextToSpeech("Wähle eine Datei aus diesem Verzeichnis mit den Laut- und Leisetasten aus. Bestätige deine Auswahl mit der OK-Taste.")
                                HspList = get_HoerspieleImVerzeichnis(CurPlayDirname)
                                if(HspList):
                                        CurPlayFilename = HspList[0]
                                        CurFileNr = -1
#                                        HLP_TextToSpeech("Verzeichnis: '" + CurPlayDirname + "'")
#                                        HLP_TextToSpeech("Erste Datei im Verzeichnis: '" + CurPlayFilename + "'")
#                                        HLP_PlayFile(CurPlayFilename, True)
                                else:
                                        HLP_PlayAdminFile("0401_error.mp3")
                                        HLP_TextToSpeech("Das Verzeichnis '" + CurPlayDirname + "' ist leer.")
                                        HLPMode = Mode_Idle
                                        allLED(False)
                        elif (not ChoosePlayMode == PlayMode_Idle):
                                if (os.path.isfile(FilenameRFIDDatabase)):
                                        RFIDLinks = readDatabase("RFID")
                                if (WithDebugMsg):
                                        print(RFIDLinks)
                                print("CurPlayDirname:" + CurPlayDirname)
                                RFIDLinks[LastCardId] = {"PlayMode": str(ChoosePlayMode), "PlayName": CurPlayDirname}
                                if (WithDebugMsg):
                                        print(RFIDLinks)
                                writeDatabase("RFID", RFIDLinks)
                                HLP_PlayAdminFile("0400_tag_ok.mp3")
                                HLP_TextToSpeech("Ich habe die Karte mit dem Ordner verknüpft. Lege eine Karte auf.")
                                HLPMode = Mode_Idle
                                allLED(False)
                elif (RegisterMode == RegisterMode_File):
                        if (os.path.isfile(FilenameRFIDDatabase)):
                                RFIDLinks = readDatabase("RFID")
                        RFIDLinks[LastCardId] = {"PlayMode": str(ChoosePlayMode), "PlayName": CurPlayFilename}
                        writeDatabase("RFID", RFIDLinks)
                        HLP_PlayAdminFile("0400_tag_ok.mp3")
                        HLP_TextToSpeech("Ich habe die Karte mit der Datei verknüpft. Lege eine Karte auf.")
                        HLPMode = Mode_Idle
                        allLED(False)

def do_ButtonPlayHeld():
        global HLPMode, RegisterMode
        global bounceHeldPlay
        bounceHeldPlay = True
        print("Button Play/OK lange gedrückt.")
        if (HLPMode == Mode_Delete):
                HLP_PlayAdminFile("0802_reset_aborted.mp3")
                HLPMode = Mode_Idle
                HLP_TextToSpeech("Löschen einer Karte abgebrochen.")
        elif (HLPMode == Mode_Play) or (HLPMode == Mode_Idle):
                HLP_PlayAdminFile("0800_reset_tag.mp3")
                HLPMode = Mode_Delete
                HLP_TextToSpeech("RFID-Karte löschen. Bitte zu löschende Karte auflegen.")
                print("HLPMode = " + HLPModes[HLPMode])
        elif (HLPMode == Mode_Register):
                HLP_PlayAdminFile("0802_reset_aborted.mp3")
                HLPMode = Mode_Play
                HLP_TextToSpeech("Vorgang abgebrochen.")

def do_ButtonUp():
        global HLPMode, RegisterMode, ChoosePlayMode, VolumeCur
        global CurPlayDirname, CurDirNr, CurDir_list, CurPlayFilename, CurFileNr, HspList, PathToAdminAudio, PathToPlayAudio
        global bounceHeldUp
        if (bounceHeldUp):
                bounceHeldUp = False
                return
        print("Button Up gedrückt.")
        print("HLPMode = " + HLPModes[HLPMode])
        if (HLPMode == Mode_Delete):
                do_AbortDelete()
        elif (HLPMode == Mode_Idle or HLPMode == Mode_Play):
                VolumeCur += 5
                if (VolumeCur > VolumeMax):
                        VolumeCur = VolumeMax
                HLP_Volume(str(VolumeCur))
        elif (HLPMode == Mode_Register):
                if (RegisterMode == RegisterMode_Dir):
                        #Nächstes Verzeichnis auswählen und die erste Datei daraus abspielen
                        print("CurDir_list hat " + str(len(CurDir_list)) + " Einträge")
                        print("CurDirNr: " + str(CurDirNr))
                        if ((CurDirNr + 1) < len(CurDir_list)):
                                CurDirNr += 1
                        else:
                                CurDirNr = 0
                        CurPlayDirname = CurDir_list[CurDirNr]
                        print("Nächstes Verzeichnis (" + str(CurDirNr) + ") anspielen")
                        HspList = get_HoerspieleImVerzeichnis(CurPlayDirname)
                        HLP_PlayFile(HspList[0])
                elif (RegisterMode == RegisterMode_File):
                        #Nächste Datei auswählen und abspielen
                        print("Nächste Datei anspielen")
                        if ((CurFileNr + 1) < len(HspList)):
                                CurFileNr += 1
                        else:
                                CurFileNr = 0
                        CurPlayFilename = HspList[CurFileNr]
                        HLP_PlayFile(HspList[CurFileNr])
                elif (RegisterMode == RegisterMode_PlayMode):
                        ChoosePlayMode += 1
                        if (ChoosePlayMode > PlayMode_Max):
                                ChoosePlayMode = 1
                        do_PlayAdminMenu(ChoosePlayMode)

def do_ButtonUpHeld():
        global CurFileNr
        global bounceHeldUp
        bounceHeldUp = True
        print("Button Up Held gedrückt.")
        if (HLPMode == Mode_Play):
                do_FastForward()

def get_PlayDirectories():
        # Die zurückgegebene Liste enthält nicht den Basispfad, hier '/home/pi/Hoerspielland-Play/HLP-Audio'
        dir_list = sorted(os.listdir(PathToPlayAudio), key=str.casefold)
        CurDir_list = []
        for folder in dir_list:
                if (os.path.isdir(PathToPlayAudio + "/" + folder)):
                        CurDir_list.append(folder)
        return CurDir_list

def get_HoerspieleImVerzeichnis(folder):
        # folder ist das Unterverzeichnis im Basispfad, hier '/home/pi/Hoerspielland-Play/HLP-Audio'
        # auch die Antwort enthält nicht den Basispfad
        # Aufgelistet werden die Hörspiele, die entweder als mp3-Datei oder als weiteres Unterverzeichnis im Verzeichnis liegen
        # HspList enthält also entweder Dateien
        # also z.B. 'Serienname/01 - Folge1.mp3' und/oder 'Serienname/01 - Folge1/'
        dir_list = sorted(os.listdir(PathToPlayAudio + "/" + folder), key=str.casefold)
        HspList = []
        for filename in dir_list:
                if (os.path.isfile(PathToPlayAudio + "/" + folder + "/" + filename)):
                        file_parts = os.path.splitext(filename)
                        if (file_parts[1]):
                                endung = file_parts[1]
                                endung = endung.lower()
                                if (endung.casefold() == ".mp3" or endung.casefold() == ".wav" or endung.casefold() == ".flac"):
                                        HspList.append(folder + "/" + filename)
                elif (os.path.isdir(PathToPlayAudio + "/" + folder + "/" + filename)):
                        # Im Serien-Verzeichnis liegt ein Unterverzeichnis. Prüfen, ob darin mp3-Dateien liegen.
                        # Wenn ja, wird das Unterverzeichnis der Liste hinzugefügt.
                        u_dir_list = sorted(os.listdir(PathToPlayAudio + "/" + folder + "/" + filename), key=str.casefold)
                        for u_filename in u_dir_list:
                                if (os.path.isfile(PathToPlayAudio + "/" + folder + "/" + filename + "/" + u_filename)):
                                        u_file_parts = os.path.splitext(u_filename)
                                        if (u_file_parts[1]):
                                                u_endung = u_file_parts[1]
                                                u_endung = u_endung.lower()
                                                if (u_endung.casefold() == ".mp3" or u_endung.casefold() == ".wav" or u_endung.casefold() == ".flac"):
                                                        HspList.append(folder + "/" + filename)
                                                        break
        return HspList


#Taster: 11 (BCM.17, GPIO.0), 13 (BCM.27, GPIO.2), 15 (BCM.22, GPIO.3)
# Angabe mit BOARDzahl = physikalischer Pin auf dem Pi-Board
buttonDown = Button("BOARD11", hold_time=4)
buttonPlay = Button("BOARD13", hold_time=4)
buttonUp = Button("BOARD15", hold_time=4)

buttonDown.when_released = do_ButtonDown
buttonDown.when_held = do_ButtonDownHeld
buttonPlay.when_released = do_ButtonPlay
buttonPlay.when_held = do_ButtonPlayHeld
buttonUp.when_released = do_ButtonUp
buttonUp.when_held = do_ButtonUpHeld

ledDown = LED("BOARD29")
ledPlay = LED("BOARD31")
ledUp = LED("BOARD36")

def opposite(device):
    for value in device.values:
        yield not value

def allLED(stat):
        global ledDown, ledPlay, ledUp, tLED
        if (stat):
                ledDown.source = opposite(buttonDown)
                ledPlay.source = opposite(buttonPlay)
                ledUp.source = opposite(buttonUp)
                tLED = time.time()
        else:
                ledDown.source = buttonDown
                ledPlay.source = buttonPlay
                ledUp.source = buttonUp
                tLED = 0

def blinkAllLED():
        global ledOown, ledPlay, ledUp
        ledDown.source = None
        ledPlay.source = None
        ledUp.source = None
        ledDown.blink(0.5, 0.5, None, True)
        ledPlay.blink(0.5, 0.5, None, True)
        ledUp.blink(0.5, 0.5, None, True)

allLED(False)

reader = SimpleMFRC522()
# Durch SimpleMFRC522() wird die GPIO-Bezeichnung auf BOARD gesetzt, also physikalische Pins

HLP_InitPlayer()
tRFID = 0
tOLED = time.time()
tLED = time.time()

RFIDLinks = readDatabase("RFID")
LastCardId = ""

try:
        print("Warte auf RFID-Karte")
        while (True):
                id, text = reader.read_no_block()
                if (not id):
                        # Aufrufe von der WebApp annehmen
                        try:
                                data, addr = WebApp_UDP_Socket.recvfrom( 1024 )
                        except:
                                data = False
                        if (data):
                                dataStr = data.decode("unicode_escape")
                                print("Aufruf über UDP von der WebApp: " + dataStr)
                                if ("#btnPlay" in dataStr):
                                        do_ButtonPlay()
                                elif ("#btnDownHeld" in dataStr):
                                        do_ButtonDownHeld()
                                elif ("#btnUpHeld" in dataStr):
                                        do_ButtonUpHeld()
                                elif ("#btnDown" in dataStr):
                                        do_ButtonDown()
                                elif ("#btnUp" in dataStr):
                                        do_ButtonUp()
                                elif ("hlp-spotify-folge:" in dataStr):
                                        print("Spotify-Hörspiel direkt abspielen:")
                                        HLPMode = Mode_Play
                                        PlayName = dataStr
                                        HLP_TextToSpeech("Spiele Datei '" + PlayName + "' direkt (ohne RFID).")
                                        HLP_PlayFile(PlayName)
                                else:
                                        try:
                                                id = int(dataStr)
                                        except:
                                                id = False
                if (id):
                        if (LastCardId != str(id)):
                                allLED(True)

                                LastCardId = str(id)
                                print("Karte erkannt: ", id)
                                RFIDLinks = readDatabase("RFID")
                                if (RFIDLinks.get(LastCardId, {}).get("PlayMode", {}) and RFIDLinks.get(LastCardId, {}).get("PlayName")):
                                        print("Die Karte ist bekannt: PlayMode = " + PlayModes[int(RFIDLinks[LastCardId]["PlayMode"])] + " / PlayName = " + RFIDLinks[LastCardId]["PlayName"])
                                        print("HLPMode = " + HLPModes[HLPMode])
                                        if (HLPMode == Mode_Delete):
                                                del RFIDLinks[LastCardId]
                                                writeDatabase("RFID", RFIDLinks)
                                                HLP_PlayAdminFile("0801_reset_tag_ok.mp3")
                                                HLP_TextToSpeech("Die Karte wurde gelöscht.")
                                                HLPMode = Mode_Idle
                                                showLogo()
                                                allLED(False)
                                        else:
                                                if (HLPMode == Mode_Play and PlayMode == PlayMode_Album):
                                                        HLP_PlaylistFortschritt()

                                                HLPMode = Mode_Play
                                                PlayModeStr = RFIDLinks[LastCardId]["PlayMode"]
                                                PlayName = RFIDLinks[LastCardId]["PlayName"]
                                                PlayMode = int(PlayModeStr)
                                                
                                                print("PlayMode = " + PlayModes[PlayMode] + " / PlayName = " + PlayName)
                                                if (PlayMode == PlayMode_ShuffleDir):
                                                        HLP_TextToSpeech("Spiele Shuffle-Datei [" + PlayName + "].")
                                                        HLP_PlayShuffle(PlayName)
                                                elif (PlayMode == PlayMode_Album):
                                                        HLP_TextToSpeech("Spiele Verzeichnis/Album '" + PlayName + "'.")
                                                        HLP_PlayAlbum(PlayName)
                                                elif (PlayMode == PlayMode_File):
                                                        HLP_TextToSpeech("Spiele Datei '" + PlayName + "'.")
                                                        HLP_PlayFile(PlayName)
                                                else:
                                                        print("Auf der Karte ist ein unbekannter PlayMode gespeichert (" + str(PlayMode) + ")")
                                                        allLED(False)
                                else:
                                        HLP_PlayAdminFile("0300_new_tag.mp3")
                                        HLP_TextToSpeech("Oh! Eine neue Karte. Lasse die Karte während der Auswahl liegen. Wähle zuerst das Verzeichnis mit den Laut- und Leisetasten aus. Bestätige deine Auswahl mit der OK-Taste. Halte die OK-Taste lange gedrückt, um abzubrechen.")
                                        RFIDLinks[LastCardId] = {"PlayMode": "", "PlayName": ""}
                                        writeDatabase("RFID", RFIDLinks)
                                        HLPMode = Mode_Register
                                        RegisterMode = RegisterMode_Dir
                                        showMsg("", "Neue Karte:", "Verzeichnis wählen")
                                        CurDir_list = get_PlayDirectories()
                                        if (CurDir_list):
                                                CurPlayDirname = CurDir_list[0]
                                                HspList = get_HoerspieleImVerzeichnis(CurPlayDirname)
                                                if(HspList):
                                                        CurPlayFilename = HspList[0]
                                                        CurDirNr = -1
                                                        CurFileNr = 0
        #                                                HLP_TextToSpeech("Verzeichnis: '" + CurPlayDirname + "'")
        #                                                HLP_TextToSpeech("Erste Datei im Verzeichnis: '" + CurPlayFilename + "'")
        #                                                HLP_PlayFile(CurPlayFilename, True)
                                                else:
                                                        HLP_PlayAdminFile("0401_error.mp3")
                                                        HLP_TextToSpeech("Das Verzeichnis '" + CurPlayDirname + "' ist leer.")
                                                        HLPMode = Mode_Idle
                                                        allLED(False)
                                        else:
                                                HLP_PlayAdminFile("0401_error.mp3")
                                                HLP_TextToSpeech("Es gibt noch keine Dateien im Verzeichnis '" + PathToPlayAudio + "'.")
                                                HLPMode = Mode_Idle
                                                allLED(False)
                        tRFID = time.time()
                elif (tRFID + 3 < time.time()):
                        if (HLPMode == Mode_Register):
                                HLP_PlayAdminFile("0802_reset_aborted.mp3")
                                HLPMode = Mode_Idle
                                HLP_TextToSpeech("Registrierung abgebrochen. Die Karte muss während der gesamten Registrierung liegen bleiben.")
                                allLED(False)
                        LastCardId = ""
                        tRFID = time.time()

                t = time.time()

                if (tOLED + 120 < t and (tOLED + 120 + 10) > t):
                        # Timeout: nach 2 Minuten Nichtstun, wird das Display auf das Logo umgeschaltet
                        showLogo()
                        if (HLPMode == Mode_Register):
                                HLPMode = Mode_Idle

                if (tOLED + ClearScreenAfter < t and (tOLED + ClearScreenAfter + 10) > t):
                        # Bildschirmschoner: nach x Minuten Pause, wird das Display auf Schwarz umgeschaltet
                        showClear()

                if (tNextShow > 0 and tNextShow + 4 < t):
                        showToggle()

                if (tLED > 0 and tLED + 3600 < t):
                        allLED(False)
                        tLED = 0


                time.sleep(1.2)
finally:
        print("Ende")
